# -*- coding: utf-8 -*-
from __future__ import absolute_import
import datetime

from django import forms
from django.conf import settings
from django.contrib import admin, messages
from django.core.urlresolvers import reverse
from django.db import connection, DatabaseError, models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from cms.models.pagemodel import Page
from cms.models.titlemodels import Title

from jmb.core.admin import AjaxInline, register_inline, ConstrainedModelForm
from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.i18n.models import Language

import acomea.event.forms as acomea_forms
from acomea.event.models import (
    Event, Invite,
    EventTranslation, GreetingsTranslation,
    EventTemplate, EventReminderTranslation, EventQuestion,
    EventContactQuestion, EventConfirmTranslation,
    InviteAnswer)
from acomea.event.tasks import SendReminderTask, ReSendInvitesTask
from acomea.event.admin_forms import EventActionForm


##############################
# FORMSETS                   #
##############################
class TranslationInlineBaseFormset(forms.models.BaseInlineFormSet):
    def __init__(self, *args, **kwargs):
        super(TranslationInlineBaseFormset, self).__init__(
            *args, **kwargs
        )
        instance = kwargs.pop(
            'instance', None
        )
        if instance:
            try:
                self.language = instance.default_language
            except Language.DoesNotExist:
                self.language = None
            except DatabaseError:
                connection._rollback()


##############################
# INLINE TRANSLATION CLASSES #
##############################
class EventTranslationInline(admin.StackedInline):
    model = EventTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': (
                'language', 'title',
                'body_image', 'body',
                'email_from', 'template',
            ),
            'classes': ('',)
        }),
    )


class EventReminderTranslationInline(admin.StackedInline):
    model = EventReminderTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': (
                'language', 'body_image', 'body', 'template',
            ),
            'classes': ('',)
        }),
    )


class EventConfirmTranslationInline(admin.StackedInline):
    model = EventConfirmTranslation
    form = acomea_forms.EventConfirmTranslationForm
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': (
                'language', 'email_subject', 'body_image', 'body', 'template',
            ),
            'classes': ('',)
        }),
    )


class GreetingsTranslationInline(admin.StackedInline):
    model = GreetingsTranslation
    extra = 0
    formset = TranslationInlineBaseFormset
    fieldsets = (
        (None, {
            'fields': (
                'language', 'body', 'invite_status',
            ),
            'classes': ('',)
        }),
    )


##############################
# INLINE TRANSLATION CLASSES #
##############################

class EventQuestionAjaxInline(AjaxInline):
    model = EventQuestion
    fk_name = 'event'
    list_display = (
        'ordering', 'get_link_enabled', 'get_link_required', 'position', 'language', 'get_category',
        'question', 'get_question_qtype', 'get_choices', 'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )
    width = 800
    height = 600


class EventContactQuestionAjaxInline(AjaxInline):
    model = EventContactQuestion
    fk_name = 'event'
    list_display = (
        'ordering', 'get_link_enabled', 'get_link_required',
        'question', 'contact_field', 'get_edit_icon_iframe', 'get_delete_icon_iframe',
    )
    width = 800
    height = 600


class EventQuestionForm(ConstrainedModelForm):
    model = EventQuestion
    hidden_fields = ('event',)


class EventContactQuestionForm(ConstrainedModelForm):
    model = EventContactQuestion
    hidden_fields = ('event',)


class EventQuestionModelAdmin(ExtendibleModelAdmin):
    model = EventQuestion
    form = EventQuestionForm
    list_display = (
        'event', 'question', 'language',
        'get_edit_icon',
        'get_delete_icon',
    )
    fieldsets = (
        ('Additional information', {
            'fields': (
                'date_create', 'date_last_modify', 'creator', 'last_modifier'
            ),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'event', 'question', 'enabled', 'required', 'position', 'language', 'ordering'
        )}),
    )


class EventContactQuestionModelAdmin(ExtendibleModelAdmin):
    class Media:
        js = (
            "event/js/admin_dajaxice.js",
        )

    model = EventContactQuestion
    form = EventContactQuestionForm
    list_display = (
        'event', 'contact_field',
        'get_edit_icon',
        'get_delete_icon',
    )
    fieldsets = (
        ('Additional information', {
            'fields': (
                'date_create', 'date_last_modify', 'creator', 'last_modifier'
            ),
            'classes': ('collapse',)
        }),
        (None, {'fields': (
            'event', 'question', 'contact_field', 'enabled', 'required', 'ordering'
        )}),
    )


class EventAdmin(ExtendibleModelAdmin):
    class Media:
        js = (
            "event/js/admin_dajaxice.js",
            "event/js/collapsed_stacked_inlines.js",
            "event/js/prevent_close.js",
            "/jsi18n/?app=jmb.core",
            'jmb/js/jquery.ui.js',
            'jmb/js/jquery-ui-sliderAccess.js',
            "jmb/js/jquery-ui-timepicker-addon.js",
        )
        css = {
            'all': ("jmb/css/jquery-ui-timepicker-addon.css",)
        }

    prepopulated_fields = {"slug": ("name",)}
    list_display = (
        'get_status_display',
        'event_date_start', 'event_date_end', 'event_adhesion_deadline',
        'name', 'slug', 'landing_page',
        'get_edit_icon',
        'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('main'), {
            'fields': (
                'status', 'name', 'slug',
                ('landing_page', 'default_language',),
                ('event_date_start', 'event_date_end',),
                'event_adhesion_deadline',
            ),
        }),
    )
    inlines = (
        EventTranslationInline,
        EventReminderTranslationInline,
        EventConfirmTranslationInline,
        GreetingsTranslationInline,
    )
    tabs = (
        (_('main'), {'items': [_('main')], 'active': True}),
        (_('presentation'), {'items': [EventTranslation]}),
        (_('questions'), {'items': [EventQuestionAjaxInline, EventContactQuestionAjaxInline]}),
        (_('greetings'), {'items': [GreetingsTranslation]}),
        (_('reminder'), {'items': [EventReminderTranslation]}),
        (_('confirm'), {'items': [EventConfirmTranslationInline]}),
        (_('logs'), {'items': [_('logs admin')]}),
    )
    save_on_top = False

    # ACTIONS
    action_form = EventActionForm
    action_form_fields = {
        'resend_invites': ['send_from_date', 'send_to_date'],
        'resend_invites_no_controls': ['send_from_date', 'send_to_date'],
        'send_reminder': ['send_from_date', 'send_to_date'],
        'send_reminder_no_controls': ['send_from_date', 'send_to_date'],
    }
    actions = [
        'resend_invites',
        'resend_invites_no_controls',
        'send_reminder',
        'send_reminder_no_controls',
    ]

    def resend_invites(self, request, queryset):
        GET = request.GET.copy()
        REQ = request.REQUEST.copy()
        pks = request.POST.getlist('_selected_action')
        select_across = int(request.POST.get('select_across'))
        msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
        messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
        date_from = request.POST.get('send_from_date', '').strip()
        date_from = datetime.datetime.strptime(date_from, '%d/%m/%Y %H:%M') if date_from else None
        date_to = request.POST.get('send_to_date', '').strip()
        date_to = datetime.datetime.strptime(date_to, '%d/%m/%Y %H:%M') if date_to else None
        res = ReSendInvitesTask.delay_or_fail(
            app_label='event',
            model=self.model,
            pks=pks,
            select_across=select_across,
            get=GET,
            req=REQ,
            date_from=date_from,
            date_to=date_to
        )

    resend_invites.short_description = _("Resend event invites")

    def resend_invites_no_controls(self, request, queryset):
        GET = request.GET.copy()
        REQ = request.REQUEST.copy()
        pks = request.POST.getlist('_selected_action')
        select_across = int(request.POST.get('select_across'))
        msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
        messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
        date_from = request.POST.get('send_from_date', '').strip()
        date_from = datetime.datetime.strptime(date_from, '%d/%m/%Y %H:%M') if date_from else None
        date_to = request.POST.get('send_to_date', '').strip()
        date_to = datetime.datetime.strptime(date_to, '%d/%m/%Y %H:%M') if date_to else None
        res = ReSendInvitesTask.delay_or_fail(
            app_label='event',
            model=self.model,
            pks=pks,
            select_across=select_across,
            get=GET,
            req=REQ,
            check_can_create=False,
            date_from=date_from,
            date_to=date_to
        )

    resend_invites_no_controls.short_description = _("Resend event invites (NO CONTROLS)")

    def send_reminder(self, request, queryset):
        GET = request.GET.copy()
        REQ = request.REQUEST.copy()
        pks = request.POST.getlist('_selected_action')
        select_across = int(request.POST.get('select_across'))
        msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
        messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
        date_from = request.POST.get('send_from_date', '').strip()
        date_from = datetime.datetime.strptime(date_from, '%d/%m/%Y %H:%M') if date_from else None
        date_to = request.POST.get('send_to_date', '').strip()
        date_to = datetime.datetime.strptime(date_to, '%d/%m/%Y %H:%M') if date_to else None
        res = SendReminderTask.delay_or_fail(
            app_label='event',
            model=self.model,
            pks=pks,
            select_across=select_across,
            get=GET,
            req=REQ,
            date_from=date_from,
            date_to=date_to
        )

    send_reminder.short_description = _("Send  event reminder")

    def send_reminder_no_controls(self, request, queryset):
        GET = request.GET.copy()
        REQ = request.REQUEST.copy()
        pks = request.POST.getlist('_selected_action')
        select_across = int(request.POST.get('select_across'))
        msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
        messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
        date_from = request.POST.get('send_from_date', '').strip()
        date_from = datetime.datetime.strptime(date_from, '%d/%m/%Y %H:%M') if date_from else None
        date_to = request.POST.get('send_to_date', '').strip()
        date_to = datetime.datetime.strptime(date_to, '%d/%m/%Y %H:%M') if date_to else None
        res = SendReminderTask.delay_or_fail(
            app_label='event',
            model=self.model,
            pks=pks,
            select_across=select_across,
            get=GET,
            req=REQ,
            check_can_create=False,
            date_from=date_from,
            date_to=date_to
        )

    send_reminder_no_controls.short_description = _("Send  event reminder (NO CONTROLS)")

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "landing_page":
            titles = Title.objects.filter(
                slug=settings.ACOMEA_EVENT_INVITE_PAGE_SLUG,
            )
            kwargs['queryset'] = Page.objects.none()
            if titles:
                title = titles[0]
                page = title.page
                kwargs['queryset'] = page.get_children().filter(published=True)
        return super(EventAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


class InviteAnswerInline(admin.TabularInline):
    model = InviteAnswer
    extra = 0
    fieldsets = (
        (_('main'), {
            'fields': (
                ('get_question', 'answer',)
            ),
        }),
    )
    readonly_fields = ('get_question',)
    formfield_overrides = {
        models.TextField: {'widget': forms.TextInput},
    }

    def has_add_permission(self, request):
        return False


class InviteAdmin(ExtendibleModelAdmin):
    class Media:
        js = ("event/js/admin_dajaxice.js",)

    form = acomea_forms.InviteAdminForm

    list_display = (
        'get_status_display',
        'sent', 'event', 'is_trace_request',
        'get_contact_last_name',
        'get_contact_first_name',
        'get_contact_email',
        'step',
        'get_answers_output',
        'get_linklandingpage',
        'get_edit_icon', 'get_delete_icon',
    )
    list_display_csv = (
        'get_status_display', 'sent', 'event',
        'get_contact_last_name', 'get_contact_first_name', 'get_contact_email',
        'step', 'get_raw_answers_output',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status', 'event',
                'contact', 'sent', 'is_trace_request', 'step',
            )
        }),
    )
    advanced_search_fields = (
        ('id:Id', 'sent'),
        ('contact__first_name__icontains:First Name', 'contact__last_name__icontains:Last Name'),
        ('contact__emails__email__icontains:Email',),
        ('event__in',),
        ('status', 'step__in'),
    )
    readonly_fields = ('is_trace_request',)

    inlines = (InviteAnswerInline,)

    def get_contact_first_name(self, obj):
        return u"{}".format(obj.contact.first_name)
    get_contact_first_name.short_description = _("First name")
    get_contact_first_name.admin_order_field = 'contact__first_name'

    def get_contact_last_name(self, obj):
        return u"{}".format(obj.contact.last_name)
    get_contact_last_name.short_description = _("Last name")
    get_contact_last_name.admin_order_field = 'contact__last_name'

    def get_contact_email(self, obj):
        return u"{}".format(obj.contact.get_default_email())
    get_contact_email.short_description = _("Email")

    def get_contact_details(self, obj):
        return "%s %s (%s)" % (
            obj.contact.first_name, obj.contact.last_name, obj.contact.get_default_email()
        )
    get_contact_details.short_description = _("Contact details")


class EventTemplateAdmin(ExtendibleModelAdmin):
    list_display = (
        'name',
        'get_edit_icon', 'get_delete_icon',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'name', 'file',
            )
        }),
    )


register_inline(EventQuestionAjaxInline)
register_inline(EventContactQuestionAjaxInline)

admin.site.register(Event, EventAdmin)
admin.site.register(Invite, InviteAdmin)
admin.site.register(EventTemplate, EventTemplateAdmin)
admin.site.register(EventQuestion, EventQuestionModelAdmin)
admin.site.register(EventContactQuestion, EventContactQuestionModelAdmin)
