# -*- coding: utf-8 -*-

from exceptions import ValueError

from django import forms
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.db.models.loading import get_model
from django.db.models.query import QuerySet
from django.core.exceptions import ValidationError
from django.forms import fields_for_model
from django.template import Context, loader
from django.template.loader import get_template

from django.utils.translation import ugettext_lazy as _

from web.utils.generic import get_acomea_host, get_acomea_tcp_protocol

from jmb.core.middleware.thread_local import get_request
from jmb.survey.models import Question
from jmb.survey.forms import QuestionForm

from acomea.contact.models import Contact
from acomea.event.models import InviteAnswer, EventQuestion, InviteContactAnswer
from acomea.profilo.models import AcomeaProfilo


# AdhesionForm
class AdhesionForm(QuestionForm):
    def __init__(self, *args, **kwargs):
        self.invite = kwargs.pop('invite')
        self.event = self.invite.event
        self.contact = self.invite.contact
        self.contact_related_managers = [rel.get_accessor_name() for rel in Contact._meta.get_all_related_objects()]
        super(AdhesionForm, self).__init__(*args, **kwargs)
        if self.contact and self.event:
            self.set_question_fields()

    def set_question_fields(self):
        self.set_event_questions()
        # Controllo se effettuare le domande di contatto in base allo stato rac se esiste profilo acomea per contatto
        do_contact_questions = True
        stati_cliente = ['6', '9']
        profiles = self.contact.profiles.all()
        if profiles:
            profilo = profiles[0]
            if profilo.rac.stato in stati_cliente:
                do_contact_questions = False
        if do_contact_questions:
            self.set_contact_questions()

    def set_event_questions(self):
        event_questions = self.event.questions.filter(
            enabled=True,
        ).order_by('ordering')
        for event_question in event_questions:
            key = "question_%d" % event_question.id
            field_class, widget_class = self.get_form_question_widget_class(event_question.question)
            self.fields[key] = field_class
            self.fields[key].label = event_question.question.question.capitalize()
            self.fields[key].empty_label = None
            self.fields[key].required = event_question.required
            self.set_field_initial(key, event_question.question)
            self.fields[key].help_text = None
            self.fields[key].group = "event_%s" % event_question.position

    def set_contact_questions(self):
        event_contact_questions = self.event.contact_questions.filter(
            enabled=True
        ).order_by('ordering')
        for event_contact_question in event_contact_questions:
            field = event_contact_question.contact_field
            if field not in self.contact_related_managers:
                if field == "auth_materiale_commerciale" and self.contact.auth_materiale_commerciale:
                    pass
                else:
                    key = "contact_question__%s" % field
                    self.fields[key] = fields_for_model(Contact)[field]
                    self.fields[key].initial = getattr(self.invite.contact, field)
                    if event_contact_question.required:
                        self.fields[key].required = True
                    else:
                        self.fields[key].required = True if self.fields[key].required else False
                    self.fields[key].help_text = None
                    self.fields[key].group = "contact"
                    self.fields[key].label = event_contact_question.question
            else:
                # Recupero il modello utilizzato dalla related
                related_model = getattr(Contact, field).related.model
                related_app_model_name = "%s__%s" % (related_model._meta.app_label, related_model.__name__.lower())
                # Recupero row del related manager per impostare initial sui campi
                contact_related_manager_row = None
                if "default" in [f.name for f in related_model._meta.fields]:
                    try:
                        contact_related_manager_row = getattr(self.contact, field).get(default=True)
                    except related_model.DoesNotExist:
                        pass
                if not contact_related_manager_row:
                    contact_related_manager_rows = getattr(self.contact, field).all()
                    if len(contact_related_manager_rows):
                        contact_related_manager_row = contact_related_manager_rows[0]
                # Creo un campo del form per ogni campo del related manager
                for related_model_field in fields_for_model(
                        related_model,
                        exclude=[
                            'contact', 'default',
                            'date_create', 'date_last_modify',
                            'status', 'ordering', 'creator', 'last_modifier',
                        ]
                ):
                    key = "contact_question__%s__%s" % (related_app_model_name, related_model_field)
                    self.fields[key] = fields_for_model(related_model)[related_model_field]
                    if contact_related_manager_row:
                        related_manager_row_value = getattr(contact_related_manager_row, related_model_field)
                        if related_manager_row_value:
                            if related_manager_row_value.__class__.__name__ == 'ManyRelatedManager':
                                self.fields[key].initial = related_manager_row_value.all()
                            else:
                                self.fields[key].initial = related_manager_row_value
                    if event_contact_question.required:
                        self.fields[key].required = True
                    else:
                        self.fields[key].required = True if self.fields[key].required else False
                    self.fields[key].help_text = None
                    self.fields[key].group = "%s__%s" % (field, related_app_model_name)

    def set_field_initial(self, key, question):
        try:
            answer = self.invite.answers.get(question=question)

            if question.qtype == "T":
                self.fields[key].initial = answer.answer

            elif question.qtype == "A":
                self.fields[key].initial = answer.answer

            elif question.qtype == "R":
                previous_answers = question.choices.filter(choice__in=answer.answer.split("|"))
                self.fields[key].initial = previous_answers[0].id

            elif question.qtype == "S":
                previous_answers = question.choices.filter(choice__in=answer.answer.split("|"))
                self.fields[key].initial = previous_answers[0].id

            elif question.qtype == "C":
                previous_answers = question.choices.filter(choice__in=answer.answer.split("|"))
                self.fields[key].initial = previous_answers

        except InviteAnswer.DoesNotExist:
            self.fields[key].initial = None

    def get_form_fields(self):
        form_fields_dict = [dict(
            group=field.group,
            field=forms.forms.BoundField(self, self.fields[key], key)
        ) for key, field in self.fields.iteritems()]
        return form_fields_dict

    def save(self, status):

        # Elimino tutte le risposte già associate all'invito
        self.invite.answers.all().delete()
        self.invite.contact_answers.all().delete()

        if status == settings.ACOMEA_EVENT_INVITE_ACCEPTED:

            # Ciclo su ogni gruppo usato per raggruppare i fields del form
            groups_set = set([unicode(field.group) for (key, field) in self.fields.iteritems()])
            for group_name in groups_set:
                form_group_fields = [dict(
                    key=key,
                    field=field
                ) for key, field in self.fields.iteritems() if field.group == group_name]

                # Validazione su ogni singolo campo del gruppo event che corrisponde ad un istanza di InviteAnswer
                if group_name.startswith("event"):
                    for element in [k for k in form_group_fields if k['key'] in self.cleaned_data]:
                        event_question_id = element['key'].split("_")[-1]
                        event_question = EventQuestion.objects.get(id=event_question_id)
                        answer = self.cleaned_data[element['key']]
                        if isinstance(answer, QuerySet):
                            answer = "|".join(val['choice'] for val in answer.values())
                        if answer and answer != "":
                            try:
                                invite_answer = InviteAnswer(
                                    invite=self.invite,
                                    question=event_question.question,
                                    answer=answer,
                                )
                                invite_answer.save()
                            except (ValidationError, ValueError) as e:
                                print e

                # Validazione su ogni singolo campo del gruppo contact che corrisponde ad un campo di Contact
                elif group_name == "contact":
                    for element in form_group_fields:
                        field_key = element['key'].replace("contact_question__", "")
                        field_value = self.cleaned_data[element['key']]
                        if self.validate_contact_answer(field_value) and element['key'] in self.cleaned_data:
                            try:
                                # Eseguo update sul contatto
                                setattr(self.contact, field_key, field_value)
                                self.contact.save()
                                # Memorizzo la risposta in InviteContactAnswer
                                invite_contact_answer = InviteContactAnswer(
                                    invite=self.invite,
                                    contact_field=field_key,
                                    answer=field_value,
                                )
                                invite_contact_answer.save()
                            except (ValidationError, ValueError) as e:
                                print e

                # validazione su gruppi rimanenti che corrispondono ad un record di una tabella collegata a Contact
                else:

                    # Memorizzo nel modello utilizzato dalla relate tramite get_or_create
                    not_done_fields = []
                    model_instance_dict = dict()
                    field_name, app_name, model_name = group_name.split("__")
                    model = get_model(app_name, model_name, seed_cache=False)
                    model_instance_dict["contact"] = self.contact
                    for element in form_group_fields:
                        field_key = element['key'].replace("contact_question__", "").split("__")[-1]
                        field_value = self.cleaned_data[element['key']]
                        if field_value and element['key'] in self.cleaned_data:
                            if self.fields[element['key']].required and field_value.__class__.__name__ != "QuerySet":
                                model_instance_dict[field_key] = field_value
                            else:
                                not_done_fields.append(dict(field_key=field_key, key=element['key']))
                    model_instance, created = model.objects.get_or_create(**model_instance_dict)
                    for field_dict in not_done_fields:
                        model_instance_dict[field_dict["field_key"]] = self.cleaned_data[field_dict["key"]]
                        setattr(model_instance, field_dict["field_key"], self.cleaned_data[field_dict["key"]])
                    model_instance.save()

                    # Memorizzo in InviteContactAnswer
                    answer = ""
                    for key, value in model_instance_dict.iteritems():
                        answer += "%s: %s \n" % (key, value)
                    invite_contact_answer = InviteContactAnswer(
                        invite=self.invite,
                        contact_field=field_name,
                        answer=answer,
                    )
                    invite_contact_answer.save()

        self.update_invite_status(status)

    def validate_contact_answer(self, value):
        if isinstance(value, bool) and value is None:
            return False
        elif isinstance(value, str):
            if value is None or value.strip() == "":
                return False
        return True

    def update_invite_status(self, status):
        # Aggiornamento informazioni invito
        self.invite.status = status
        if status == settings.ACOMEA_EVENT_INVITE_ACCEPTED:
            template = settings.CONTACT_CMSAPPS_ACCEPT_INVITE_EMAIL_TEMPLATE
            subject = "Invito ad evento %s accettato da %s %s" % (
                self.invite.event.name,
                self.invite.contact.first_name,
                self.invite.contact.last_name,
            )
            self.invite.step = "greetings"
            if self.invite.event.confirm_translations.count():
                self.invite.send_confirm_email()
        elif status == settings.ACOMEA_EVENT_INVITE_DECLINED:
            template = settings.CONTACT_CMSAPPS_DECLINE_INVITE_EMAIL_TEMPLATE
            subject = "Invito ad evento %s declinato da %s %s" % (
                self.invite.event.name,
                self.invite.contact.first_name,
                self.invite.contact.last_name,
            )
        self.invite.save()
        # INVIO EMAIL
        self.send_mail(template, subject)

    def send_mail(self, template, subject):
        html_template = get_template(template)
        request = get_request()
        protocol = get_acomea_tcp_protocol(request)
        ctx_dict = Context({
            'contact': self.invite.contact,
            'event': self.invite.event,
            'static': protocol + "://" + get_acomea_host() + settings.STATIC_URL,
            'invite_answers': self.invite.get_answers(),
            'contact_answers': self.invite.contact_answers.all(),
        })
        from_email = settings.ACOMEA_EVENT_EMAIL_SENT_FROM
        to = settings.ACOMEA_EVENT_INVITE_EMAIL_TO
        html_message = html_template.render(ctx_dict)
        msg = EmailMultiAlternatives(subject, "", from_email, to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()


# GreetingsForm
class GreetingsForm(forms.Form):
    pass