# -*- coding: utf-8 -*-

from django.conf import settings
from django.core import serializers
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _

from jmb.core.utils.functions import import_class
from jmb.core.middleware.force_response import ForceResponse

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from acomea.event.models import Invite
from acomea.event.cms.models import AdhesionSettings, GreetingsSettings
from acomea.contact.utils.mailer import render_template_from_string


# Root class to inherit
class AbstractPluginPlugin(CMSPluginBase):
    fields = (
        'status', 'invite_status_to_be_visible', 'template_name', 'form'
    )

    # Imposta se sessione vuota ed il plugin è il primo
    def _set_session_step_first_plugin(self, session, plugin_dict):
        if not 'acomea_event_step' in session:
            session['acomea_event_step'] = ''
            if plugin_dict['is_first']:
                session['acomea_event_step'] = plugin_dict.get(
                    'plugin'
                ).get_plugin_name().lower()

    def _get_list_plugins(self, plugins):
        list_plugins = []
        for plugin in plugins:
            list_plugins.append(plugin)
        return list_plugins

    def _get_plugin_positions(self, instance):
        plugins = instance.placeholder.cmsplugin_set.all().order_by('position')
        list_plugins = self._get_list_plugins(plugins)
        plugin = instance.cmsplugin_ptr
        position = list_plugins.index(plugin)
        previous_plugin = ""
        next_plugin = ""
        is_first = False
        is_last = False

        if position == 0:
            is_first = True
        else:
            previous_plugin = list_plugins[position-1]

        if position == len(list_plugins)-1:
            is_last = True
        else:
            next_plugin = list_plugins[position+1]

        plugin_dict = {
            'plugins': list_plugins,
            'plugin': plugin,
            'position': position,
            'is_first': is_first,
            'is_last': is_last,
            'previous_plugin': previous_plugin,
            'next_plugin': next_plugin
        }
        return plugin_dict


# Adhesion plugin class (event adhesion step)
class AdhesionPlugin(AbstractPluginPlugin):
    module = _('Event')
    model = AdhesionSettings
    name = _('Adhesion')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)

        # Check if user is coming from tokenized view
        if 'acomea_event_invite_id' not in session:
            messages.error(request, _("Accedere all'area inviti seguendo il link ricevuto nella mail"))
            raise ForceResponse(HttpResponseRedirect('/'))

        invite = Invite.objects.get(pk=session['acomea_event_invite_id'])

        # Redirect to homepage if invite status is setted yet
        if invite.status is not None:
            raise ForceResponse(HttpResponseRedirect('/'))

        # Set if session is empty and plugin is first
        if not 'acomea_event_step' in session:
            session['acomea_event_step'] = ''
            if plugin_dict['is_first']:
                session['acomea_event_step'] = plugin_dict['plugin'].get_plugin_name().lower()

        # Check if step is the actual one
        if session['acomea_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            form_class = import_class(instance.form)

            contact = invite.contact
            event = invite.event
            adhesion = event.translations.get(
                language__id=event.default_language.id,
            )
            description = render_template_from_string(
                dict(contact=contact, invite=invite, is_email=False),
                adhesion.body
            )
            if request.method == 'POST':
                plugin_form = form_class(invite=invite)
                # If back button was pressed
                if 'adhesion_reset' in request.POST and not plugin_dict['is_first']:
                    session['acomea_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                    self.render_template = "event/cms_plugins/empty.html"
                # If accept button was pressed
                if 'adhesion_accept' in request.POST:
                    plugin_form = form_class(request.POST, invite=invite)
                    if plugin_form.is_valid():
                        plugin_form.save(settings.ACOMEA_EVENT_INVITE_ACCEPTED)
                        session['acomea_event_step'] = "greetings"
                        self.render_template = "event/cms_plugins/empty.html"
                # If decline button was pressed
                elif 'adhesion_decline' in request.POST:
                    plugin_form.save(settings.ACOMEA_EVENT_INVITE_DECLINED)
                    session['acomea_event_step'] = "greetings"
                    self.render_template = "event/cms_plugins/empty.html"
            else:
                plugin_form = form_class(invite=invite)
            context.update({
                'object': instance,
                'adhesion_form': plugin_form,
                'fields': plugin_form.get_form_fields(),
                'placeholder': placeholder,
                'session': session,
                'adhesion': adhesion,
                'description': description,
                'plugin_dict': plugin_dict,
                'event': event,
            })
        return context
plugin_pool.register_plugin(AdhesionPlugin)


# Greetings plugin class
class GreetingsPlugin(AbstractPluginPlugin):
    module = _('Event')
    model = GreetingsSettings
    name = _('Greetings')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        session = request.session
        plugin_dict = self._get_plugin_positions(instance)

        # Check if user is coming from tokenized view
        if 'acomea_event_invite_id' not in session:
            messages.error(request, _("Accedere all'area inviti seguendo il link ricevuto nella mail"))
            raise ForceResponse(HttpResponseRedirect('/'))

        invite = Invite.objects.get(pk=session['acomea_event_invite_id'])

        # inserisco in sesione i plugin presenti in 'Content_Central_Inside'
        session['acomea_event_plugins'] = serializers.serialize("json", plugin_dict['plugins'])

        # Check if step is the actual one
        if session['acomea_event_step'] != instance.invite_step_to_be_visible:
            self.render_template = "event/cms_plugins/empty.html"

        # Check if step can be show according to invite status
        elif instance.invite_status_to_be_visible and instance.invite_status_to_be_visible != invite.status:
            self.render_template = "event/cms_plugins/empty.html"

        else:
            self.render_template = instance.template_name
            self._set_session_step_first_plugin(session, plugin_dict)
            form_class = import_class(instance.form)

            contact = invite.contact
            event = invite.event

            greetings_translations = event.greetings_translations.filter(
                language__id=event.default_language.id,
                invite_status=invite.status,
            )

            description = ""
            if greetings_translations.count():
                greetings_translation = greetings_translations[0]
                description = render_template_from_string(
                    dict(contact=contact, invite=invite),
                    greetings_translation.body
                )

            # If back button was pressed
            if (
                request.method == 'POST' and
                'greetings_reset' in request.POST and
                not plugin_dict['is_first']
            ):
                session['acomea_event_step'] = plugin_dict['previous_plugin'].get_plugin_name().lower()
                self.render_template = "event/cms_plugins/empty.html"

            # If home button was pressed
            if request.method == 'POST' and 'greetings_home' in request.POST:
                plugin_form = form_class(request.POST)
                if plugin_form.is_valid():
                    raise ForceResponse(HttpResponseRedirect('/'))

            else:
                plugin_form = form_class()

            context.update({
                'object': instance,
                'presentation_form': plugin_form,
                'placeholder': placeholder,
                'session': session,
                'description': description,
                'plugin_dict': plugin_dict
            })

        return context
plugin_pool.register_plugin(GreetingsPlugin)
