# -*- coding: utf-8 -*-
from __future__ import absolute_import
import datetime

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.http import int_to_base36

from cms.models.pagemodel import Page

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from ckeditor.fields import RichTextField as HTMLFieldTinyMCE
from jmb.core.utils.token import TokenGenerator
from jmb.i18n.models import Language
from jmb.survey.models import Answer, Question


class ActiveEventManager(models.Manager):
    def get_query_set(self):
        return super(ActiveEventManager, self).get_query_set().filter(
            status=1, event_adhesion_deadline__gte=datetime.date.today())


class Event(DateModel, UserModel, StatusModel, OrderedModel, SEOModel):
    name = models.CharField(
        max_length=100,
        unique=True,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        max_length=100,
        unique=True,
        verbose_name=_('slug'),
    )
    event_date_start = models.DateField(
        verbose_name=_('start date'),
    )
    event_date_end = models.DateField(
        verbose_name=_('end date')
    )
    event_adhesion_deadline = models.DateField(
        verbose_name=_('adhesion deadline')
    )
    landing_page = models.ForeignKey(
        Page,
        related_name="events",
        verbose_name=_('landing page'),
        help_text=_(
            "Page must be published and "
            "non-root to appear here"
        )
    )
    default_language = models.ForeignKey(
        Language,
        related_name="events",
        verbose_name=_("default language")
    )

    def get_default_template_path(self):
        translations = self.translations.filter(
            language=self.default_language,
        )
        if translations:
            return translations[0].template.file.path
        return self.translations.all()[0].template.file.path

    def get_default_title(self):
        translations = self.translations.filter(
            language=self.default_language,
        )
        if translations:
            return translations[0].title
        return self.translations.all()[0].title

    # def invite_contact(self, contacts=None):
    #     contacts = contacts if isinstance(contacts, list) else Contact.objects.all()
    #     for contact in contacts:
    #         contact.invite(self)

    def expired(self):
        today = datetime.date.today()
        return self.event_date_start <= today

    def save(self, *args, **kwargs):
        add_questions = True if self.pk is None else False
        super(Event, self).save(*args, **kwargs)
        if add_questions:
            for question in Question.objects.filter(status=1):
                event_question = EventQuestion(
                    event=self,
                    question=question,
                    language=question.language,
                    enabled=question.default,
                    required=question.required,
                )
                event_question.save()

    class Meta(object):
        verbose_name = _('event')
        verbose_name_plural = _('events')
        ordering = ('-event_date_start', 'name')

    def __unicode__(self):
        return self.name


class ActiveEvent(Event):
    objects = ActiveEventManager()

    class Meta:
        proxy = True
        verbose_name = _('active event')
        verbose_name_plural = _('active events')


class EventTemplate(UserModel, DateModel, OrderedModel):
    name = models.CharField(
        _("Name"),
        unique=True,
        max_length=255,
    )
    file = models.FileField(
        upload_to=settings.ACOMEA_EVENT_TEMPLATE_PATH,
        verbose_name=_("Template")
    )

    class Meta:
        ordering = ('ordering', 'name')
        verbose_name = _("Template")
        verbose_name_plural = _("Templates")

    def __unicode__(self):
        return self.name


class EventTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="translations",
        verbose_name=_("event")
    )
    body_image = models.FileField(
        upload_to=settings.ACOMEA_EVENT_BODY_IMAGE_UPLOAD_TO,
        blank=True, null=True,
        verbose_name=_('body image'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    title = models.CharField(
        max_length=100,
        verbose_name=_('title'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
    )
    email_from = models.EmailField(
        verbose_name=_('email from'),
        default=getattr(settings, 'ACOMEA_EVENT_TRANSLATION_DEFAULT_EMAIL', '')
    )
    template = models.ForeignKey(
        EventTemplate,
        verbose_name=_('template'),
        related_name='translations',
    )
    note = models.TextField(
        null=True, blank=True,
        verbose_name=_('note'),
    )
    css = models.TextField(
        null=True, blank=True,
        verbose_name=_('css'),
    )

    class Meta(object):
        verbose_name = _('event translation')
        verbose_name_plural = _('event translations')
        ordering = ('-event__event_date_start', 'event__name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"%s: %s" % (
            self.language,
            self.event.name
        )


class EventReminderTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="reminder_translations",
    )
    name = models.CharField(
        max_length=100,
        #unique=True,
        verbose_name=_('name'),
    )
    body_image = models.FileField(
        upload_to=settings.ACOMEA_EVENT_BODY_IMAGE_UPLOAD_TO,
        blank=True, null=True,
        verbose_name=_('body image'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
    )
    template = models.ForeignKey(
        EventTemplate,
        verbose_name=_('template'),
        related_name='reminder_translations',
        blank=True, null=True,
    )

    def get_template_path(self):
        if self.template:
            return self.template.file.path
        return None

    class Meta(object):
        verbose_name = _('reminder translation')
        verbose_name_plural = _('reminder translations')
        ordering = ('name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"%s: %s" % (
            self.name,
            self.language,
        )


class EventConfirmTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="confirm_translations",
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_('name'),
    )
    email_subject = models.CharField(
        max_length=100,
        verbose_name=_('email subject'),
    )
    body_image = models.FileField(
        upload_to=settings.ACOMEA_EVENT_BODY_IMAGE_UPLOAD_TO,
        blank=True, null=True,
        verbose_name=_('body image'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
        help_text="Use {{ invite.get_print_answers }} to print contact answers",
    )
    template = models.ForeignKey(
        EventTemplate,
        verbose_name=_('template'),
        related_name='confirm_translations',
        blank=True, null=True,
    )

    def get_template_path(self):
        if self.template:
            return self.template.file.path
        return None

    class Meta(object):
        verbose_name = _('confirm translation')
        verbose_name_plural = _('confirm translations')
        ordering = ('name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"%s: %s" % (self.name, self.language)


class EventQuestion(DateModel, UserModel, OrderedModel):
    event = models.ForeignKey(
        'Event', related_name="questions",
        verbose_name=_('event')
    )
    question = models.ForeignKey(
        Question, related_name="event_questions",
        verbose_name=_('question')
    )
    language = models.ForeignKey(
        Language,
        related_name='questions'
    )
    enabled = models.BooleanField(
        verbose_name=_("enabled"),
        default=True,
    )
    required = models.BooleanField(
        verbose_name=_("required"),
        default=True,
    )
    position = models.CharField(
        max_length=100,
        choices=settings.ACOMEA_EVENT_EVENTQUESTION_POSITION_CHOICES,
        verbose_name=_('Position')
    )

    class Meta(object):
        verbose_name = _('event question')
        verbose_name_plural = _('event questions')
        ordering = ('language__id', 'question__ordering', 'question__question')

    def __unicode__(self):
        return u"%s %s %s" % (self.event.name, self.language, self.question)

    def get_category(self):
        return self.question.category

    get_category.short_description = _("category")

    def get_choices(self):
        return self.question.get_choices()

    get_choices.short_description = _("choices")

    def get_question_qtype(self):
        return self.question.get_qtype_display()

    get_question_qtype.short_description = _("qtype")

    def get_link_enabled(self):
        return """
            <a id='acomea_event_eventquestion_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.enabled else 'no'))

    get_link_enabled.short_description = _('Set')
    get_link_enabled.allow_tags = True

    def get_link_required(self):
        return """
            <a id='acomea_event_requiredquestion_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.required else 'no'))

    get_link_required.short_description = _('Req')
    get_link_required.allow_tags = True


class AdhesionTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="adhesion_translations",
    )
    name = models.CharField(
        max_length=100,
        #unique=True,
        verbose_name=_('name'),
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
    )

    class Meta(object):
        verbose_name = _('adhesion translation')
        verbose_name_plural = _('adhesion translations')
        ordering = ('name', 'language')
        unique_together = ('event', 'language')

    def __unicode__(self):
        return u"%s: %s" % (
            self.name,
            self.language,
        )


class GreetingsTranslation(DateModel, UserModel, StatusModel, OrderedModel):
    event = models.ForeignKey(
        Event,
        related_name="greetings_translations"
    )
    invite_status = models.IntegerField(
        choices=settings.ACOMEA_EVENT_INVITE_STATUS_CODES,
        verbose_name=_('Invite status')
    )
    language = models.ForeignKey(
        Language,
        verbose_name=_('language'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('body'),
    )

    class Meta(object):
        verbose_name = _('greetings translation')
        verbose_name_plural = _('greetings translations')
        ordering = ('event__name', 'language')
        unique_together = ('event', 'invite_status', 'language')

    def __unicode__(self):
        return u"%s: %s" % (
            self.language,
            self.event
        )

    def get_body(self):
        return self.body

    #get_landingpage.short_description = _("LP")
    get_body.allow_tags = True


class Invite(DateModel, UserModel, OrderedModel):
    status = models.IntegerField(
        choices=settings.ACOMEA_EVENT_INVITE_STATUS_CODES,
        null=True, blank=True,
        verbose_name=_('status')
    )
    event = models.ForeignKey(
        Event,
        related_name='invites',
        verbose_name=_('event')
    )
    contact = models.ForeignKey(
        'contact.Contact',
        related_name='invites',
        verbose_name=_('contact')
    )
    step = models.CharField(
        max_length=20,
        default='adhesion',
        choices=settings.ACOMEA_EVENT_INVITE_STEP_CHOICES,
        verbose_name=_('step')
    )
    sent = models.BooleanField(
        default=False,
        verbose_name=_('sent'),
    )
    is_trace_request = models.BooleanField(
        default=False,
        verbose_name=_('is trace request'),
    )

    def get_code(self):
        return "%s" % self.id

    def make_token(self):
        token_generator = TokenGenerator()
        return token_generator.make_token(self.get_code())

    def make_base36(self):
        return int_to_base36(self.id)

    def check_token(self, token):
        token_generator = TokenGenerator()
        return token_generator.check_token(self.get_code(), token)

    def get_answers(self):
        answers = []
        for event_question in self.event.questions.all().order_by('ordering'):
            try:
                invite_answer = self.answers.get(question=event_question.question)
                app = dict(
                    question=event_question.question.question,
                    answer=invite_answer.answer,
                )
                answers.append(app)
            except InviteAnswer.DoesNotExist:
                continue
        return answers

    def get_page_url(self):
        return """/event/%s/%s-%s/""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )

    def get_linklandingpage(self):
        return """<a href=/event/%s/%s-%s/ target="_blank">LINK</a>""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )

    get_linklandingpage.short_description = _("LP")
    get_linklandingpage.allow_tags = True

    def get_landingpage(self):
        return """/event/%s/%s-%s/""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )

    get_landingpage.short_description = _("URL")
    get_landingpage.allow_tags = True

    def get_landingpage_text_url(self):
        return """<a href=/event/%s/%s-%s/ target="_blank">Cliccare qui per confermare la partecipazione</a>""" % (
            self.step,
            self.make_base36(),
            self.make_token()
        )

    def get_landingpage_with_contact_flag_text_url(self):
        analytics = "firstAccess=true" if self.contact.user else "newContact=true"
        return """<a href=/event/%s/%s-%s/?%s target="_blank">Cliccare qui per confermare la partecipazione</a>""" % (
            self.step,
            self.make_base36(),
            self.make_token(),
            analytics
        )

    def get_default_template_path(self):
        translations = self.event.translations.filter(
            language=self.event.default_language,
        )
        if translations:
            return translations[0].template.file.path
        return self.event.translations.all()[0].template.file.path

    def get_default_reminder_template_path(self):
        translations = self.event.reminder_translations.filter(
            language=self.event.default_language,
        )
        if translations:
            return translations[0].template.file.path
        return self.event.reminder_translations.all()[0].template.file.path

    def get_default_confirm_template_path(self):
        translations = self.event.confirm_translations.filter(
            language=self.event.default_language,
        )
        if translations:
            return translations[0].template.file.path
        return self.event.confirm_translations.all()[0].template.file.path

    def get_default_title(self):
        translations = self.event.translations.filter(
            language=self.event.default_language,
        )
        if translations:
            return translations[0].title
        return self.event.translations.all()[0].title

    def get_answers_output(self):
        text = ""
        for answer in self.get_answers():
            text += "<strong>%s</strong>: %s" % (
                answer["question"], answer["answer"]
            )
            text += "<br/>"
        for contact_answer in self.contact_answers.all():
            text += "<strong>%s</strong>: %s\n" % (
                contact_answer.contact_field,
                contact_answer.answer,
            )
            text += "<br/>"
        return text

    get_answers_output.short_description = _("Answers")
    get_answers_output.allow_tags = True

    def get_raw_answers_output(self):
        text = ""
        for answer in self.get_answers():
            text += "%s %s" % (answer["question"], answer["answer"])
            text += " - "
        for contact_answer in self.contact_answers.all():
            text += "%s %s" % (contact_answer.contact_field, contact_answer.answer)
            text += " - "
        text = text.rstrip(" - ")
        return text

    get_raw_answers_output.short_description = _("Answers")

    def get_print_answers(self):
        from django.utils.safestring import mark_safe

        html = "<table>"
        for answer in self.get_answers():
            html += "<tr><td><b>%s:<b></td><td>%s</td></tr>" % (answer["question"], answer["answer"])
        for contact_answer in self.contact_answers.all():
            html += "<tr><td>%s</td><td>%s</td></tr>" % (contact_answer.contact_field, contact_answer.answer)
        html += "</table>"
        return mark_safe(html)

    def send_confirm_email(self):
        from jmb.mailer.sender import Mailer
        from acomea.contact.utils.mailer import render_template_from_string

        if self.status == settings.ACOMEA_EVENT_INVITE_ACCEPTED:
            event = self.event
            contact = self.contact
            test = True
            confirm_translation = event.confirm_translations.get(
                language__id=event.default_language.id,
            )
            mailer = Mailer(
                self, test=test, test_in_subject=False, is_confirm=True,
                subject=confirm_translation.email_subject,
            )
            description = render_template_from_string(
                dict(event=event, contact=contact, invite=self),
                confirm_translation.body
            )
            context = dict(description=description, invite=self)
            if confirm_translation.body_image:
                context["top_image_url"] = "%s://%s/%s" % (
                    settings.PROFILO_TCP_PROTOCOL,
                    settings.PROFILO_HOST,
                    confirm_translation.body_image.url.lstrip("/")
                )
            mail_created, email_object = mailer.create_message(
                contact, context, check_can_create=False
            )
            if mail_created and test:
                mailer.send(email_object)

    class Meta(object):
        verbose_name = _('invite')
        verbose_name_plural = _('invites')
        unique_together = ('event', 'contact')
        ordering = ('contact__last_name',)

    def __unicode__(self):
        return "%s: %s %s - %s" % (
            self.event.name,
            self.contact.first_name, self.contact.last_name,
            self.get_status_display()
        )


class InviteAnswer(Answer):
    invite = models.ForeignKey(
        Invite,
        related_name='answers',
        verbose_name=_('invite')
    )

    def __unicode__(self):
        return "%s - %s: %s? %s" % (self.invite.event.name, self.invite.contact, self.question, self.answer)

    class Meta(object):
        verbose_name = _('invite answer')
        verbose_name_plural = _('invite answers')
        unique_together = ('invite', 'question')

    def get_question(self):
        return self.question


class InviteContactAnswer(DateModel, UserModel):
    invite = models.ForeignKey(
        Invite,
        related_name='contact_answers',
        verbose_name=_('invite')
    )
    contact_field = models.CharField(
        max_length=100,
        choices=settings.ACOMEA_EVENT_CONTACT_QUESTION_CONTACT_FIELD_CHOICES,
        verbose_name=_('contact field')
    )
    answer = models.TextField(
        verbose_name=_('answer text')
    )
    submission_date = models.DateTimeField(
        auto_now=True,
        verbose_name=_("submission date"),
    )

    def __unicode__(self):
        return "%s: %s? %s" % (self.invite, self.contact_field, self.answer)

    class Meta(object):
        verbose_name = _('invite contact answer')
        verbose_name_plural = _('invites contact answers')
        unique_together = ('invite', 'contact_field')


class EventContactQuestion(DateModel, UserModel, OrderedModel):
    event = models.ForeignKey(
        'Event',
        related_name="contact_questions",
        verbose_name=_('event')
    )
    question = HTMLFieldTinyMCE(
        verbose_name=_('question label')
    )
    contact_field = models.CharField(
        max_length=100,
        choices=settings.ACOMEA_EVENT_CONTACT_QUESTION_CONTACT_FIELD_CHOICES,
        verbose_name=_('contact field')
    )
    enabled = models.BooleanField(
        verbose_name=_("enabled"),
        default=True,
    )
    required = models.BooleanField(
        verbose_name=_("required"),
        default=True,
    )

    class Meta(object):
        verbose_name = _('event contact question')
        verbose_name_plural = _('event contact questions')
        ordering = ('ordering',)

    def __unicode__(self):
        return u"%s: %s" % (self.event, self.contact_field)

    def get_link_enabled(self):
        return """
            <a id='acomea_event_contact_question_enabled_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.enabled else 'no'))

    get_link_enabled.short_description = _('Set')
    get_link_enabled.allow_tags = True

    def get_link_required(self):
        return """
            <a id='acomea_event_contact_question_required_%s' href='#'>
                <img alt="False" src="/static/admin/img/icon-%s.gif">
            </a>""" % (self.id, ('yes' if self.required else 'no'))

    get_link_required.short_description = _('Req')
    get_link_required.allow_tags = True


if settings.CMS_ENABLED:
    import acomea.event.cms.plugins